const grip = require('grip');

// IMPORTANT!!!
// `routes` file should have something like this
// * localhost:80,over_http
// if you use Docker to run pushpin be sure to pass `target` environment var
// with a value of localhost:8080

exports.register = function(server, options, next) {
  let events = 'application/websocket-events';

  // do this since hapi does not support websocket-events content type
  server.ext('onRequest', function(request, reply) {
    // set the rel content-type in a new header if needed on the future
    if (events === request.headers['content-type']) {
      request.headers['content-type'] = 'text/html';
      request.headers['x-content-type'] = events;
    }

    return reply.continue();
  });

  const openConnection = function(connectionId) {
    // we use connectionId because is the way graphql is managing subscriptions on ui clients.
    let message = grip.webSocketControlMessage('subscribe', {
      'channel': connectionId,
    });

    return [
      new grip.WebSocketEvent('OPEN'),
      new grip.WebSocketEvent('TEXT', 'c:' + message),
    ];
  };

  const closeConnection = function(connectionId) {
    const message = grip.webSocketControlMessage('unsubscribe', {
      'channel': connectionId,
    });

    outEvents = [
      new grip.WebSocketEvent('CLOSE'),
      new grip.WebSocketEvent('TEXT', 'c:' + message),
    ];

    return outEvents;
  };

  server.route({
    method: 'POST',
    path: '/ws',
    handler: (request, reply) => {
      const inEvents = grip.decodeWebSocketEvents(request.payload);
      const connectionId = request.headers['connection-id'];

      let payload, outEvents = [];
      let allOutEvents = [];

      try {
        for (let i = 0; i < inEvents.length; i++) {
          const type = inEvents[i].getType();

          if (type === 'OPEN') {
            // Open the WebSocket:
            outEvents = openConnection(connectionId, reply); //
          } else if (type === 'TEXT') {
            // Get content from event
            payload = inEvents[i].getContent();
            // parse content
            try {
              payload = JSON.parse(payload);
            } catch (e) {
              let message = {
                type: 'error',
                message: e.message,
              };

              outEvents = [
                new grip.WebSocketEvent('TEXT', JSON.stringify(message)),
              ];
            }

            const keepAliveMessage = grip.webSocketControlMessage(
                'keep-alive',
                {content: JSON.stringify({type: 'ack'}), timeout: 30}, // used to keep alive your client connection
            );

            // here you should parse payload and do whatever you wish/need
            server.log(['debug', 'websocket'], `payload event type is ${payload.type}`);

            outEvents = [
              new grip.WebSocketEvent('TEXT', JSON.stringify(`payload event type is ${payload.type}`)),
              keepAliveMessage,
            ];
          } else if (type === 'CLOSE' || type === 'DISCONNECT') {
            outEvents = closeConnection(connectionId);
          } else {
            outEvents = [
              new grip.WebSocketEvent('TEXT', JSON.stringify('not a valid event type')),
            ];
            server.log(['debug', 'websocket'], `${inEvents[i].getType()} is not a valid event type`);
          }

          allOutEvents = allOutEvents.concat(outEvents);
        }

        return reply(grip.encodeWebSocketEvents(allOutEvents))
            .header('Sec-WebSocket-Extensions', 'grip; message-prefix=""')
            // this one is something used by GraphQL Subscriptions so I don't think you need that but
            // right now I don't remember.
            // .header('Sec-WebSocket-Protocol', 'graphql-ws')
            .type('application/websocket-events');
      } catch (e) {
        return reply(e);
      }
    },
  });

  next();
};

exports.register.attributes = {
  name: 'ws-over-http',
};
